from logger import Logger


class GestureTracker(Logger):
    """
    Abstract class for gesture tracker definitions. Contains methods for
    changing the state of touch points.
    """
    # Supported gesture classes
    supported_gestures = NotImplemented

    # Supported gesture types (filled on factory registration)
    gesture_types = []

    # Configurable properties (see configure() method)
    configurable = []

    def __init__(self, area):
        self.area = area

    def handle_event(self, event):
        """
        Handle an event that was delegated by a area. The tracker
        implementation should define a handler function for the event.
        Otherwise, the event will be ignored.
        """
        handler_name = 'on_' + event.get_type()

        if hasattr(self, handler_name):
            getattr(self, handler_name)(event)

    def trigger(self, gesture):
        gesture.set_tracker(self)
        self.info('Triggered %s.' % gesture)
        self.area.handle_gesture(gesture)

    def configure(self, **kwargs):
        for name, value in kwargs.iteritems():
            if name not in self.configurable:
                raise ValueError('%s.%s is not a configurable property.'
                                 % (self.__class__.__name__, name))

            setattr(self, name, value)


class Gesture(object):
    """
    Abstract class that represents a triggered gesture.
    """
    _type = NotImplemented

    def __init__(self, originating_event=None):
        self.tracker = None
        self.originating_event = originating_event

    def get_type(self):
        return self._type

    def get_tracker(self):
        return self.tracker

    def get_event(self):
        return self.originating_event

    def has_event(self):
        return bool(self.originating_event)

    def set_tracker(self, tracker):
        self.tracker = tracker
